@@ -26,3 +26,8 @@ def check_pandas_input(self, arg):
         raise ValueError(
             "input needs to be a numpy array or pandas data frame."
         )
+
+
+def infer_cat_feats(df, threshold=64):
+    threshold = min(len(df) // 100 + 8, threshold)
+    return df.nunique()[lambda x: x <= threshold].index.tolist()
